// simulateDoc.cpp : implementation of the CSimulateDoc class
//

#include "stdafx.h"
#include "simulate.h"

#include "simulateDoc.h"
#include "Parameters.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CConnection
void CConnection::SetConnection( int input_line  , int input_slot ,
	                             int output_line , int output_slot ,
					             unsigned long duration ,
					             unsigned long start) 
{
	m_InputLine  = input_line ;
	m_InputSlot  = input_slot ;
	m_OutputLine = output_line ;
	m_OutputSlot = output_slot ;
	m_Duration    = duration ;
	m_Start       = start    ; 
}


/////////////////////////////////////////////////////////////////////////////
// CSimulateDoc

IMPLEMENT_DYNCREATE(CSimulateDoc, CDocument)

BEGIN_MESSAGE_MAP(CSimulateDoc, CDocument)
	//{{AFX_MSG_MAP(CSimulateDoc)
	ON_UPDATE_COMMAND_UI(ID_NETWORK_PARAMETERS, OnUpdateNetworkParameters)
	ON_UPDATE_COMMAND_UI(ID_NETWORK_ST, OnUpdateNetworkSt)
	ON_UPDATE_COMMAND_UI(ID_NETWORK_TS, OnUpdateNetworkTs)
	ON_UPDATE_COMMAND_UI(ID_NETWORK_TST, OnUpdateNetworkTst)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM_IN, OnUpdateViewZoomIn)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM_OUT, OnUpdateViewZoomOut)
	ON_COMMAND(ID_NETWORK_PARAMETERS, OnNetworkParameters)
	ON_COMMAND(ID_NETWORK_ST, OnNetworkSt)
	ON_COMMAND(ID_NETWORK_TS, OnNetworkTs)
	ON_COMMAND(ID_NETWORK_TST, OnNetworkTst)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSimulateDoc construction/destruction

CSimulateDoc::CSimulateDoc()
{
	// TODO: add one-time construction code here
    m_N = 10 ;
	m_T = 15 ;
	m_interval = 1 ;
	m_type = TS ;
	m_status = new_file ;
	m_ncalls = 0 ;
	m_AveCallDuration = 1 ;
	m_AveArrivalRate = 1 ;
	m_MaxOneCallPer = min_subscriber_rate ;
	   // min_subscriber_rate is defined in Parameters.h
	m_Seed = 0 ;
	m_MenuFlag = TRUE ;
	/////////////////////////////////////////////////////////////
	m_N = 10 ;
    m_T = 8 ;
	m_interval = 6 ; 
    m_AveCallDuration = 50 ;
    m_AveArrivalRate = 2 ;
	m_MaxOneCallPer = 5 ;
	m_type = ST  ;
	        
	m_ncalls = (unsigned long)(m_AveArrivalRate * simulation_time) ;	
    m_Seed = (unsigned)time(NULL);
	srand(m_Seed);
	m_status = loaded_file ;
}

CSimulateDoc::~CSimulateDoc()
{
  
}

// RNG() is used to generate a pseudorandom integer
// from lower_limit to upper_limit
int CSimulateDoc::RNG( int lower_limit , int upper_limit )
{
  int x , y ;
  int z = (rand()%100+1) ;
  
  x = rand() % z + 1 ; // 1 <= x <= 100
  y = rand() % z + 1 ; // 1 <= y <= 100
  if (x==y) x=y-1;

  float r ;
  if (x<y)
   	r = (float)x / (float)y  ;
  else
    r = (float)y / (float)x  ;     // 0 < r < 1 

  int value = (int)((upper_limit-lower_limit+1)*r + lower_limit);
  
  return value ;
}


BOOL CSimulateDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CSimulateDoc serialization

void CSimulateDoc::Serialize(CArchive& ar)
{
	CFile* pFile = ar.GetFile();
    pFile->SeekToBegin();
	if (ar.IsStoring())
	{
		// TODO: add storing code here
		pFile->Write((int*)&m_N,sizeof(int));
		pFile->Write((int*)&m_T,sizeof(int));
		pFile->Write((int*)&m_interval,sizeof(int));

		int type ; 
		switch (m_type)
		{
		case TS  :	type = 0 ; 	break ;
		case ST  :  type = 1 ; 	break ;
		case TST :   type = 2 ; 	break ;		
		}
		pFile->Write((int*)&type,sizeof(int));
		pFile->Write((unsigned long*)&m_ncalls,sizeof(unsigned long));
	
		pFile->Write((int*)&m_AveCallDuration,sizeof(int));
		pFile->Write((int*)&m_AveArrivalRate,sizeof(int));
        pFile->Write((int*)&m_MaxOneCallPer,sizeof(int));
        pFile->Write((unsigned int*)&m_Seed,sizeof(unsigned int));
        ar.Flush() ;
	}
	else
	{
		// TODO: add loading code here
        pFile->Read((int*)&m_N,sizeof(int));
		pFile->Read((int*)&m_T,sizeof(int));
		pFile->Read((int*)&m_interval,sizeof(int));

		int type ;
		pFile->Read((int*)&type,sizeof(int));
		switch (type)
		{
		case 0 : m_type = TS ; 	break ;
		case 1 : m_type = ST ; 	break ;
		case 2 : m_type = TST  ; 	break ;
		}
		pFile->Read((unsigned long*)&m_ncalls,sizeof(unsigned long));		
	    pFile->Read((int*)&m_AveCallDuration,sizeof(int));
		pFile->Read((int*)&m_AveArrivalRate,sizeof(int));
        pFile->Read((int*)&m_MaxOneCallPer,sizeof(int));
        pFile->Read((unsigned int*)&m_Seed,sizeof(unsigned int));
    	m_status = loaded_file ;
	}

	pFile->Close();
}

/////////////////////////////////////////////////////////////////////////////
// CSimulateDoc diagnostics

#ifdef _DEBUG
void CSimulateDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CSimulateDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSimulateDoc commands

unsigned long CSimulateDoc::RandomizeNewCall(CConnection& call)
{ 
	int input_line , input_slot , output_line , output_slot ;
	int nCallDuration , nSlotsInBetween ;
	unsigned long lCallDuration , lSlotsInBetween ;
    
	output_slot = rand() % m_T ;
	input_line  = rand() % m_N ;

    nCallDuration = RNG(max((int)(m_AveCallDuration/2),1),
			                max((int)((m_AveCallDuration*3)/2),1)); ; 
	   // random duration of calls in seconds
	lCallDuration = (unsigned long)nCallDuration * sampling_rate ;
	output_line = rand() % m_N ;
      
	nSlotsInBetween = RNG(max((int)(m_T*60/2),1),    
			                   max((int)(m_T*60*3/2),1)); 
	lSlotsInBetween = (unsigned long)(nSlotsInBetween*sampling_rate/m_AveArrivalRate) ; 
 	 // random time between calls in slots
	 // Average number of slots between two calls
	 // = T * 60 * sampling_rate / m_AverageArrivalRate
	 // The factor 60 is required because Average Arrival Rate
	 // is given in calls/minute
	 // Here , the factor is made 6 , as if we have compressed
	 // the time axis
     
	input_slot  = rand() % m_T ;
			
    call.SetConnection(input_line  ,  input_slot  ,
		               output_line ,  output_slot ,
					   lCallDuration , 0 );
	return lSlotsInBetween ;
}


void CSimulateDoc::OnNetworkParameters() 
{
	// TODO: Add your command handler code here
    CParameters ParamDlg ;
	
	ParamDlg.m_N = m_N ;
	ParamDlg.m_T = m_T ;
	ParamDlg.m_interval = m_interval ;
    ParamDlg.m_frames = m_AveCallDuration ;
	ParamDlg.m_calls_per_minute = m_AveArrivalRate ;
	ParamDlg.m_MaxOneCallPer = m_MaxOneCallPer ;

	switch (m_type)
	   {
	     case 0 : ParamDlg.m_topology = TS ;  break ;
	     case 1 : ParamDlg.m_topology = ST ;  break ;
	     case 2 : ParamDlg.m_topology = TST ; break ;
	   }
	
	int response = ParamDlg.DoModal() ;  // IDOK or IDCANCEL
    if ( response == IDOK )
	{
		if ( (m_status==new_file)  ||
			 (m_status==loaded_file && 
			     (m_N!=ParamDlg.m_N || m_T!=ParamDlg.m_T 
			      || m_AveCallDuration!=ParamDlg.m_frames || m_AveArrivalRate!=ParamDlg.m_calls_per_minute
				  || m_MaxOneCallPer!=ParamDlg.m_MaxOneCallPer)  )
		   )
		{ // if new file
          //if (m_status==loaded_file && m_ncalls!=0)
		  //	  delete [m_ncalls] m_calls ;
          m_N = ParamDlg.m_N ;
		  m_T = ParamDlg.m_T ;
		  m_interval = ParamDlg.m_interval ; 
		  m_AveCallDuration = ParamDlg.m_frames ;
		  m_AveArrivalRate = ParamDlg.m_calls_per_minute ;
		  m_MaxOneCallPer = ParamDlg.m_MaxOneCallPer ;
	
		  switch (ParamDlg.m_topology)
	        {
	         case 0 : m_type = TS ;  break ;
	         case 1 : m_type = ST ;  break ;
	         case 2 : m_type = TST ; break ;
	        }		  
		  m_ncalls = (unsigned long)(m_AveArrivalRate * simulation_time) ;
    	  m_Seed = (unsigned)time(NULL);
		  srand(m_Seed);
		  m_status = loaded_file ;
    	}// end if new file
		else
		{ // else for loaded files
		  // where the parameters are the same as the old ones
          switch (ParamDlg.m_topology)
	            {
	             case 0 : m_type = TS ;  break ;
	             case 1 : m_type = ST ;  break ;
	             case 2 : m_type = TST ; break ;
	            }		  
          m_interval = ParamDlg.m_interval ;
          CString msg ;
		  msg = "The network parameters that you have just specified are the same as those specified before.\n\nWould you like to generate a new set of random connections ? \n\nIf you choose ""No"", ONLY changes made to the Network Topology and Simulation interval,if any,will be in effect.";
		  int answer =  ParamDlg.MessageBox(msg,"Warning",MB_ICONINFORMATION|MB_YESNO);
		  if ( answer!=IDNO )
		  {
			  m_N = ParamDlg.m_N ;
		      m_T = ParamDlg.m_T ;
		      m_AveCallDuration = ParamDlg.m_frames ;
			  m_AveArrivalRate = ParamDlg.m_calls_per_minute ;
		      	  
			  m_MaxOneCallPer = ParamDlg.m_MaxOneCallPer ;
			  m_ncalls = (unsigned long)(m_AveArrivalRate * simulation_time) ;    	    
              m_Seed = (unsigned)time(NULL);
		      srand(m_Seed);
          } // end else i.e. chosen YES
		} // end else i.e. for loaded files
	} // end if != OK
	UpdateAllViews(0);
}// end function

void CSimulateDoc::OnUpdateNetworkParameters(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(m_MenuFlag);
}

void CSimulateDoc::OnUpdateNetworkSt(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetRadio(m_type == ST);
	pCmdUI->Enable(m_MenuFlag);	
}

void CSimulateDoc::OnUpdateNetworkTs(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetRadio(m_type == TS);
	pCmdUI->Enable(m_MenuFlag);
}

void CSimulateDoc::OnUpdateNetworkTst(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
    pCmdUI->SetRadio(m_type == TST);
	pCmdUI->Enable(m_MenuFlag);
}

void CSimulateDoc::OnUpdateViewZoomIn(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(TRUE);
}

void CSimulateDoc::OnUpdateViewZoomOut(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(TRUE);
}


void CSimulateDoc::OnNetworkSt() 
{
	// TODO: Add your command handler code here
    m_type = ST ;	
	UpdateAllViews(0);
}

void CSimulateDoc::OnNetworkTs() 
{
	// TODO: Add your command handler code here
	m_type = TS ;
	UpdateAllViews(0);
}

void CSimulateDoc::OnNetworkTst() 
{
	// TODO: Add your command handler code here
	m_type = TST ;
	UpdateAllViews(0);
}
