// simulateDoc.h : interface of the CSimulateDoc class
//
/////////////////////////////////////////////////////////////////////////////

const unsigned long sampling_rate = 8L ;
const int simulation_time = 60 ; // 60 minutes
const int simulation_time_sec = 3600 ; // 3600 seconds
/////////////////////////////////////////////////////////////////////////////
class CConnection
{
protected :
   int  m_InputLine   ;     // inlet of a connection
   int  m_InputSlot   ;     // slot number on an inlet
   int  m_OutputLine  ;     // outlet of a connection
   int  m_OutputSlot  ;     // slot number on an outlet
   unsigned long  m_Duration ;  
                // duration of the call in slots
   unsigned long  m_Start    ;   
                // the time at which the call begins
                // this time is given in slots
                // e.g. if m_start = 1600
                // then the call begins at t=1600 slots 
                // measured from the start of simulation
                                        
public :
   CConnection()
   { m_InputLine  = -1 ;
     m_InputSlot  = -1 ;
	 m_OutputLine = -1 ;
	 m_OutputSlot = -1 ;
     m_Duration = 0 ;
	 m_Start = 0 ;
   }

   void SetConnection( int input_line  , int input_slot	 ,   
	                   int output_line , int output_slot ,
					   unsigned long duration ,  unsigned long start) ;
   void SetStart(unsigned long lStart)
   {   m_Start = lStart ; } 
   CConnection( int input_line  , int input_slot	 ,   
	            int output_line , int output_slot ,
		       unsigned long start ,  unsigned long duration) 
   {
	   	m_InputLine  = input_line ;
	    m_InputSlot  = input_slot ;
	    m_OutputLine = output_line ;
	    m_OutputSlot = output_slot ;
	    m_Duration    = duration ;
	    m_Start       = start    ; 
   }


   int GetInputLine()  { return m_InputLine  ; }
   int GetOutputLine() { return m_OutputLine ; }
   int GetInSlot()     { return m_InputSlot  ; }
   int GetOutSlot()    { return m_OutputSlot ; }
   unsigned long GetDuration()    { return m_Duration ; }
   unsigned long GetCallStart()   { return m_Start ; } 
};

enum FileStatus { new_file , loaded_file } ;
enum Topology { TS , ST , TST } ;

class CSimulateDoc : public CDocument
{
protected: // create from serialization only
	CSimulateDoc();
	DECLARE_DYNCREATE(CSimulateDoc)
// Operations

  // RNG() is used to generate a pseudorandom integer
  // from lower_limit to upper_limit .
    int RNG( int lower_limit , int upper_limit );
       
  // RandomizeNewCall() is used to randomize a new call
	unsigned long RandomizeNewCall(CConnection& call) ;
	Topology GetTopology()  { return m_type ; }
	int GetT() { return m_T ; }
	int GetN() { return m_N ; }
	unsigned int GetSeed() { return m_Seed ; }
	unsigned long GetNumCalls() {return m_ncalls;}

// Attributes
    int m_N ; // number of inlets(outlets) of the space array
	int m_T ; // number of time slots
	 // Number of input(output) subscribers is NT
	int m_interval ; // simulation interval
	Topology m_type ;
  // the last type of circuit switching topology
  // used for this document(i.e. file of a certain network)
  // For a new file , this is initialised as TST switching)
	unsigned long m_ncalls ; // number of calls(connections)
	FileStatus m_status ;
  // this indicates whether the file is newly-created
  // or have been previously saved and loaded.
	int m_AveCallDuration ;
  // the average number of seconds that a call can last for .
	int m_AveArrivalRate ;
  // the average number of calls initiated per minute
	int m_MaxOneCallPer ;
  // this means that a subscriber does at most 1 call per
  // m_MaxOneCallPer minutes...
    unsigned int m_Seed ;
  // m_Seed stores the seed of the random number generator
	BOOL m_MenuFlag ;
  // m_MenuFlag is used to disable some menu items that are meaningless
  // to be enabled during simulation , 
  // e.g. The user can not change the parameters of a neteork
  // whose operation is currently being simulated...

public:	
// Operations
	void EnableMenuFlag( BOOL flag ) 
	{ m_MenuFlag = flag ; }
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CSimulateDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CSimulateDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CSimulateDoc)
	afx_msg void OnUpdateNetworkParameters(CCmdUI* pCmdUI);
	afx_msg void OnUpdateNetworkSt(CCmdUI* pCmdUI);
	afx_msg void OnUpdateNetworkTs(CCmdUI* pCmdUI);
	afx_msg void OnUpdateNetworkTst(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewZoomIn(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewZoomOut(CCmdUI* pCmdUI);
	afx_msg void OnNetworkParameters();
	afx_msg void OnNetworkSt();
	afx_msg void OnNetworkTs();
	afx_msg void OnNetworkTst();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////
