// simulateView.h : interface of the CSimulateView class
//
/////////////////////////////////////////////////////////////////////////////

#include "simulateDoc.h"

#define NOT_USED      -1
#define ST_BUSY        2
#define ST_BLOCKED     1
#define ST_NONBLOCKED  0

// The following part of the program belongs to the TST simulation :
#define ESTABLISHED	1
#define BUSY		2
#define BLOCKED		3
#define UNDEFINED	9

#define SSLOT 		0	// source slot
#define SINLET 		1 	// source inlet
#define DSLOT  		2	// destination slot
#define DINLET 		3	// destination inlet
#define RESULT	 	4	// connection established or not

#define TRUE		1
#define FALSE		0

// There must be some macros:
#define LINE(x1,y1,x2,y2) LineWRTRect(x1,y1,x2,y2,pDC)
#define RECTANGLE(x1,y1,x2,y2) RectangleWRTRect(x1,y1,x2,y2,pDC)

// And some constants 4 graphics:
#define GR_GAP_HEIGHT   35
#define GR_SLOT_HEIGHT 19
#define GR_SLOT_WIDTH 60
#define GR_UPPER 30
#define GR_LEFT 40



class CSimulateView;

class CTimeSwitching
{
private:
	int m_T;
public:
	int *m_control_array;
	CTimeSwitching(int T);
	~CTimeSwitching();
};






















// Similar to ST_Call? yes , your right!
struct Call { int outlet ; unsigned long duration ; } ;

class CSpace
{
public:
	int m_N,m_M;		// m_N no. of inlets of the S switch
				// m_M no. of slots in the T switches
				//     connected to each inlet.

	struct Call **m_gate_select_memory;
	CSpace(int N,int M);
	int CheckDuplicateTransmitter(int Line,int Slot) ;
	~CSpace();
};

// Definitions for the TST ended!




class CST_TSI 
{
  int* to ; // time slot exchanger  
  int m_Size ; 
public :
  // default constructor
  CST_TSI(int T) ; 
  // SetValue() sets a slot in the TSI
  void SetValue(int index,int value);
  // GetValue() gets a slot in the TSI
  int  GetValue(int index);
  // CheckBusyReceiver() checks if the receiver
  // of the current call is busy
  int  CheckBusyReceiver(int key);
  // Destructor deallocates the memory of the to array
  ~CST_TSI();
};
///////////////////////////////////////////////////////////////////////////////
struct ST_Call { int outlet ; unsigned long duration ; } ;
/////////////////////////////////////////////////////////////////////////////////////
class CST_GateSelect 
{
  ST_Call** memory ;
  int m_Lines ;
  int m_Slots ;
public :
  CST_GateSelect(int T , int N) ;
  void SetOutlet(int row,int column,int out);
  int  GetOutlet(int row,int column);
  void SetDuration(int row,int column,unsigned long hold_time);
  unsigned long GetDuration(int row,int column);
  int CheckDuplicateTransmitter(int Line,int Slot) ;
  ~CST_GateSelect();
};
//////////////////////////////////////////////////////////////////////////////////////////////
class CTS_TSI
{
  int* to ;
  int m_Size ;
public :
  // default constructor
  CTS_TSI(int T) ; 
  // SetValue() sets a slot in the TSI
  void SetValue(int index,int value);
  // GetValue() gets a slot from the TSI
  int  GetValue(int index);
  // CheckDuplicateTransmitter checks whether the transmitter
  // is now communicating or not .
  int CheckDuplicateTransmitter(int key) ;
  // Destructor deallocates the memory of the to array
  ~CTS_TSI();
};

class CTS_GateSelect 
{
  ST_Call** memory ;
  int m_Lines ;
  int m_Slots ;
public :
  CTS_GateSelect(int T , int N) ;
  void SetOutlet(int row,int column,int out);
  int  GetOutlet(int row,int column);
  void SetDuration(int row,int column,unsigned long hold_time);
  unsigned long GetDuration(int row,int column);
  int CheckBusyReceiver(int Line,int Slot);
  ~CTS_GateSelect();
};
////////////////////////////////////////////////////////////////////////////////////////////
enum SimulationStatus { Simulating , Pause , NotSimulating } ;

class CSimulateView : public CScrollView
{
protected: // create from serialization only
	CSimulateView();
	DECLARE_DYNCREATE(CSimulateView)

// Attributes
  // specifies the size of the drawing area
  SIZE m_DrawingSize ;
  // specifies the step used in zooming in and out
  int m_xZoomStep ; // zooming horizontally
  int m_yZoomStep ; // zooming vertically
  float m_ZoomingRatio;

  // The following declarations belong to the TST simulator:
	CSpace *m_S;				//	 S
	CTimeSwitching **m_T[2];				// T	        T
	int TST_ServiceCall();
private:
	void WriteFrameInfo(void);
	void LineWRTRect(int x1,int y1,int x2,int y2,CDC *pDC);
	void RectangleWRTRect(int x1,int y1,int x2,int y2,CDC *pDC);
	void DrawTime(int x,int y,int col,int index);
	void UpdateTimeElement(int x,int y,int col,int line,int elementno,CDC *pDC);
	void UpdateTimeElementForAllViews(int x,int y,int col,int line,int elementno);

	void UpdateGSElementForAllViews(int x,int y,int input,int slot);
	void UpdateGSElement(int x,int y,int input,int slot,CDC *pDC);
	void DrawSpace(int x,int y);
	void DrawMUX(int x,int y);
	void DrawDEMUX(int x,int y);
	void DrawGS(int x,int y);
	
protected:

  SimulationStatus m_SimulationStatus ;
 // a pointer to the array of TSI
  CST_TSI** m_StTsi ;
 // GateSelect signifies the control memory of the space array
  CST_GateSelect* m_StGateSelect ;
 // a pointer to the array of TSI of TS topology
  CTS_TSI** m_TsTsi ;
 // GateSelect signifies the control memory of the space array
  CTS_GateSelect* m_TsGateSelect ;

  int m_CurrentSlot ;  // current slot in simulation
  unsigned long m_CurrentFrame ;
  unsigned long m_CallCount ; // holds a counter of the number of calls that have been simulated
  CConnection m_CurrentCall ; // holds the current call used in simulation 
  int m_MoreCalls ; // a flag used to indicate whether there are new calls to simulate or not
  int m_NewCall ;   // a flag used to indicate whether a new call has arrived or not
  int m_ValidCall ; // a flag used to indicate whether the current new call is valid or not
  unsigned long m_BlockNum ;
  CWinThread* m_pWinThread ;
  int m_Threading ; // a flag used to indicate whether the simulation thread 
                    // has ever been initiated                    
  int  m_YesFile ; // =1 if the user wants to output results to file
  CStdioFile* m_pFile ;
  char m_OutputFileSpec [MAX_FILE_SPEC] ;
    // the path and name of the results file;
  BOOL m_FrameAndSlot ; // =TRUE if the user wants to output the frame and slot number
  BOOL m_SrcAndDest ; // =TRUE if the user wants to output the source and destination of a call
  int m_CMContents ; 
    // output the frame contents every
    //   frame   >>   m_CMContents = 0 
    //   second  >>   m_CMContents = 1 
    //   minute  >>   m_CMContents = 2
    //   never   >>   m_CMContents = 3
public:
	CSimulateDoc* GetDocument();
	void Zoom(int in_or_out); 
	  // in_or_out = 1  >>> Zoom in
      // in_or_out = -1 >>> Zoom out
    CRect GetDimRectangle(long left,long top,long right,long bottom) ;
	void DrawLine(CDC* pDC , int x1, int y1 , int x2 , int y2 ) ;

	unsigned long MOD(unsigned long Dividend ,
					unsigned long Divisor ) ;
	// CheckNewCallArrival() sees if the current call has arrived or not yet
	int CheckNewCallArrival(unsigned long Start) ;

// ST_ServiceCall() services a call if the topology is ST
// Servicing means determining whether :
//   >>> the receiver is busy     >>> return ST_BUSY
//   >>> the call is blocked      >>> return ST_BLOCKED
//   >>> the call is non-blocked  >>> return ST_NONBLOCKED
	int ST_ServiceCall();

// ST_UpdateGateSelect() updates the duration of each call in the gate select
// AND if a call terminates it removes its data from the gate select
	void ST_UpdateGateSelect() ;
	void TST_UpdateGateSelect() ;
// EndSimulation() is used to end simulation
	void ST_EndSimulation() ;
	void TST_EndSimulation() ;

	void ST_OutputGateSelect();
	void TST_OutputGateSelect();
	int TS_ServiceCall();
	void TS_UpdateGateSelect() ;
	void TS_EndSimulation() ;
	void TS_OutputGateSelect();

    friend UINT SimulationThread(LPVOID pParam) ;
	void PrepareFirstCall() ;
    void TerminateSimulation() ;
// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CSimulateView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual void OnInitialUpdate(); // called first time after construct
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CSimulateView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CSimulateView)
	afx_msg void OnViewZoomIn();
	afx_msg void OnViewZoomOut();
	afx_msg void OnSimulateContinue();
	afx_msg void OnSimulatePause();
	afx_msg void OnSimulateStart();
	afx_msg void OnSimulateStep();
	afx_msg void OnSimulateStop();
	afx_msg void OnUpdateSimulateContinue(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSimulatePause(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSimulateStart(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSimulateStep(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSimulateStop(CCmdUI* pCmdUI);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnSimulateResultsOptions();
	afx_msg void OnUpdateSimulateResultsOptions(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in simulateView.cpp
inline CSimulateDoc* CSimulateView::GetDocument()
   { return (CSimulateDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////
